package ittp.protocol.request;

import ittp.protocol.response.Response;
import ittp.protocol.request.headers.*;
import java.rmi.RemoteException;
import ittp.ittpException;

/**
 * This class build the response to client when he send a Option request.
 * @author Frederic Bidon and Mathieu Texier
 */
public class Option extends Request {
    
    /** Creates a new instance of Option */
    public Option () {
        Id = OPTIONS;
    }
    
    /**
     * This method determine the different headers allowed and mandatory according to the permissions for a Option request.
     * After, this headers will be send to the client.
     * @param response Response send to the client.
     */
    public void process (Response response) {
        boolean is_used;
        try {
            if (!ittp.Server.getParameters ().isAllowedMethod (Id))
                throw new ittpException ("", Response.Code.ITTP_METHOD_NOT_ALLOWED);
        } catch (RemoteException e) {
            throw new ittpException (e.getMessage (), Response.Code.ITTP_INTERNAL_ERROR);
        }
        
        Range range = (Range) getHeaders (RANGE);
        if (range.isUsed ()) {
            response.getHeaders (Response.CONTENT_RANGE).setUsed (true);
            response.new Body (getFile (), range.getFirst (), range.getLast ());
        } else {
            response.getHeaders (Response.CONTENT_RANGE).setUsed (false);
            response.new Body (getFile ());
        }
        response.setVirtualBody (true);
        response.getHeaders (Response.ALLOW).setUsed (true);
        response.getHeaders (Response.ALLOW).setAllowed (true);
        
        is_used = getHeaders (CONNECTION).isUsed ();
        response.getHeaders (Response.CONNECTION).setUsed (is_used);
        
        response.getHeaders (Response.CONTENT_LENGTH).setUsed (true);
        
        is_used = getHeaders (RANGE).isUsed ();
        response.getHeaders (Response.CONTENT_RANGE).setUsed (is_used);
        
        response.getHeaders (Response.CONTENT_MD5).setUsed (true);
        response.getHeaders (Response.CONTENT_TYPE).setUsed (true);
        response.getHeaders (Response.DATE).setUsed (true);
        response.getHeaders (Response.LAST_MODIFIED).setUsed (true);
        response.getHeaders (Response.SERVER).setUsed (true);
    }
}
